package com.nisum.ascend.inventory.controller;

import com.nisum.ascend.inventory.dto.ProductDto;
import com.nisum.ascend.inventory.model.Product;
import com.nisum.ascend.inventory.repository.ProductRepository;
import com.nisum.ascend.inventory.service.ProductService;
import com.nisum.ascend.inventory.controller.ProductController;

import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.AutoConfigureWebTestClient;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.ResponseEntity;
import org.mockito.Mock;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.web.bind.annotation.ResponseBody;
import reactor.core.publisher.Mono;

import static org.junit.jupiter.api.Assertions.*;


import org.springframework.web.reactive.function.client.WebClient;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

@ExtendWith(SpringExtension.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@DirtiesContext
@AutoConfigureWebTestClient
@ActiveProfiles("test")
@Slf4j
class ProductControllerTest {

    @Autowired
    private WebTestClient client;


    @BeforeEach
    public  void setup(){

    }

    @Test
    void testProductInvalidSkuNotFound() {
        client
                .get()
                .uri("/api/products/".concat("{sku}"), "invalid")
                .exchange()
                .expectStatus()
                .isNotFound();
    }

    @Test
    void testProductBySkuFound() {
        String sku = "000001";

        client.get()
                .uri("/api/products/".concat("/{sku}"), sku)
                .exchange()
                .expectStatus()
                .isOk()
                .expectBody()
                .jsonPath("$.sku", sku);
    }


    @Test
    void testFindAllProducts(){
        webTestClient.get()
                .uri("/api/products")
                .exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_VALUE)
                .expectBodyList(Product.class)
                .consumeWith(product ->{
                    List<Product> products = product.getResponseBody();
                    products.forEach( p ->{
                        assertNotNull(p.getSku());
                        assertNotNull(p.getUpc());
                        System.out.println("Product SKU: " + p.getSku());
                        System.out.println("Product UPC: " + p.getUpc());
                    });
                });
    }


}